"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InXpresserError = exports.XpresserRepl = exports.ControllerClass = exports.getInstanceRouter = exports.getInstance = exports.init = void 0;
const ControllerClass_1 = __importDefault(require("./src/Classes/ControllerClass"));
exports.ControllerClass = ControllerClass_1.default;
const XpresserRepl_1 = __importDefault(require("./src/XpresserRepl"));
exports.XpresserRepl = XpresserRepl_1.default;
const InXpresserError_1 = __importDefault(require("./src/Errors/InXpresserError"));
exports.InXpresserError = InXpresserError_1.default;
const instanceHolder = {};
function getInstance(instanceId) {
    var _a;
    if (instanceId) {
        if (instanceId === ':keys')
            return Object.keys(instanceHolder);
        if (instanceId === ':id')
            return (_a = require('./truth')) === null || _a === void 0 ? void 0 : _a.instanceId;
        if (!instanceHolder.hasOwnProperty(instanceId))
            throw new InXpresserError_1.default(`Xpresser instanceId: ${instanceId} not found!`);
        return instanceHolder[instanceId];
    }
    else {
        if (global.hasOwnProperty('$')) {
            return global.$;
        }
        const truth = require("./truth");
        if (truth && truth.instanceId) {
            return instanceHolder[truth.instanceId];
        }
        const instances = Object.keys(instanceHolder);
        if (!instances.length)
            throw new InXpresserError_1.default(`No Xpresser instance defined found!`);
        return instanceHolder[instances[0]];
    }
}
exports.getInstance = getInstance;
function getInstanceRouter(instanceId) {
    return getInstance(instanceId).router;
}
exports.getInstanceRouter = getInstanceRouter;
function init(AppConfig, AppOptions = {}) {
    if (!global.xpresserInstance)
        global.xpresserInstance = getInstance;
    if (!global.InXpresserError)
        global.InXpresserError = InXpresserError_1.default;
    const fs = require("fs");
    const chalk = require("chalk");
    const XpresserRouter = require("@xpresser/router");
    const { Config, Options } = require("./config");
    const ObjectCollection = require("object-collection");
    const { randomStr } = require('./src/Functions/inbuilt.fn');
    const lodash = require("lodash");
    const truth = require("./truth");
    let PackageDotJson = {};
    try {
        PackageDotJson = require("./package.json");
    }
    catch (e) {
        PackageDotJson = require("../package.json");
    }
    if (AppConfig === undefined) {
        AppConfig = {};
    }
    if (AppOptions === undefined) {
        AppOptions = {};
    }
    if (!AppOptions['instanceId'])
        truth.instanceId = AppOptions['instanceId'] = randomStr(10);
    const $ = instanceHolder[AppOptions['instanceId']] = {};
    $.exit = (...args) => {
        return process.exit(...args);
    };
    if (typeof AppConfig === "string") {
        const configFile = AppConfig;
        AppConfig = {};
        if (fs.existsSync(configFile)) {
            try {
                AppConfig = require(configFile);
                if (typeof AppConfig !== "object" || (typeof AppConfig === "object" && !Object.keys(AppConfig).length)) {
                    throw new InXpresserError_1.default(`CONFIG: No exported object found in config file: (${configFile})`);
                }
            }
            catch (e) {
                console.error(e.stack);
                $.exit();
            }
        }
        else {
            console.error("Config file not found!");
            $.exit();
        }
    }
    AppConfig = AppConfig;
    const noBaseFolderDefinedError = `No base folder defined in config {paths.base}`;
    if (!AppConfig.hasOwnProperty('paths')) {
        console.log(noBaseFolderDefinedError);
        $.exit();
    }
    else {
        if (!AppConfig['paths'].hasOwnProperty('base')) {
            console.log(noBaseFolderDefinedError);
            $.exit();
        }
    }
    if (!AppConfig.hasOwnProperty('env') || !AppConfig.env) {
        console.log(`Config {env} is missing, options: (development | production | others)`);
        $.exit();
    }
    if (typeof AppConfig.env !== "string") {
        console.log(`Config {env} must be of type string!`);
        $.exit();
    }
    AppConfig = lodash.merge(lodash.clone(Config), AppConfig);
    AppOptions = lodash.merge(lodash.clone(Options), AppOptions);
    $.on = {};
    if (typeof AppConfig['ObjectCollection'] === "function") {
        const OwnObjectCollection = AppConfig.ObjectCollection();
        $.objectCollection = (obj) => new OwnObjectCollection(obj);
    }
    else {
        $.objectCollection = (obj) => new ObjectCollection(obj);
    }
    if (AppOptions.exposeDollarSign)
        global.$ = $;
    const CliConfig = global["XjsCliConfig"];
    $.config = $.objectCollection(AppConfig);
    $.options = AppOptions;
    $.engineData = $.objectCollection({});
    $.store = $.objectCollection({});
    const LaunchType = process.argv[2];
    $.engineData.set("LaunchType", LaunchType);
    if (typeof CliConfig !== "undefined" || LaunchType === "cli") {
        $.options.isConsole = true;
    }
    $.isTypescript = () => {
        return $.config.get('project.fileExtension') === ".ts";
    };
    $.isNativeCliCommand = () => {
        return LaunchType === "cli" &&
            (process.argv[3] &&
                process.argv[3].slice(0, 5) === "make:");
    };
    const enginePath = $.config.get('paths.engine');
    if (!enginePath) {
        let dirName = __dirname;
        if (["/dist", "\\dist"].includes(dirName.slice(-5))) {
            dirName = dirName.slice(0, dirName.length - 5);
        }
        $.config.set('paths.engine', `${dirName}/src/`);
    }
    require("./src/On");
    require("./src/FileEngine");
    require("./src/Extensions/Loggers");
    require("./src/Extensions/If");
    $.ifNotConsole(() => {
        $.logCalmly(`${PackageDotJson.name} version ${PackageDotJson.version}`);
        let { name, env } = $.config.all();
        if (env) {
            env = lodash.startCase(env);
            env = env.toLowerCase() === "development" ? chalk.yellow(`(${env})`) : chalk.greenBright(`(${env})`);
            env = chalk.yellow(env);
        }
        $.log(`${name} ${env}`.trim());
    });
    const timezone = $.config.get('date.timezone');
    if (timezone) {
        process.env.TZ = timezone;
    }
    require("./src/Extensions/Path");
    require("./src/global");
    const { runBootEvent } = require("./src/Events/OnEventsLoader");
    async function afterStartEvents() {
        const PluginEngine = require("./src/PluginEngine");
        const PluginData = await PluginEngine.loadPlugins(PackageDotJson);
        $.engineData.set("PluginEngineData", PluginData);
        const useDotJson = $.objectCollection();
        const useDotJsonPath = $.path.jsonConfigs("use.json");
        if ($.file.exists(useDotJsonPath)) {
            useDotJson.merge(require(useDotJsonPath));
            $.engineData.set("UseDotJson", useDotJson);
        }
        $.use = require("./src/UseEngine");
        $.router = new XpresserRouter(undefined, () => $);
        $.ifNotConsole(() => {
            require("./src/Events/Loader");
        });
    }
    $.boot = () => {
        if ($.engineData.has('hasBooted'))
            return false;
        $.engineData.set('hasBooted', true);
        return runBootEvent("start", () => {
            afterStartEvents().then(() => {
                runBootEvent("boot", () => {
                    if (!$.options.requireOnly) {
                        $.ifConsole(() => {
                            require("./src/StartConsole");
                        }, () => {
                            require("./src/StartHttp");
                        });
                    }
                });
            }).catch(e => {
                $.logError(e);
                $.logErrorAndExit('Error in $.on.boot events');
            });
        });
    };
    if ($.options.autoBoot === true) {
        $.boot();
    }
    return $;
}
exports.init = init;
