"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const express_1 = __importDefault(require("express"));
const lodash_1 = __importDefault(require("lodash"));
const http_1 = require("http");
const internals_fn_1 = require("./Functions/internals.fn");
const Path_1 = __importDefault(require("./Helpers/Path"));
const index_1 = require("../index");
const ExtendedRequestEngine_1 = __importDefault(require("./Plugins/ExtendedRequestEngine"));
const MiddlewareEngine_1 = __importDefault(require("./MiddlewareEngine"));
const ControllerService_1 = __importDefault(require("./Controllers/ControllerService"));
const ProcessServices_1 = __importDefault(require("./Controllers/ProcessServices"));
const $ = (0, index_1.getInstance)();
const $useDotJson = $.engineData.get("UseDotJson");
const AutoLoadPaths = $useDotJson.get("autoload.controllerServices", undefined);
const ServicesFolder = $.path.controllers("services");
const ServicesInMemory = $.objectCollection();
const LoadServicesInDirectory = ($folder, $files) => {
    let ServicesFolderFiles;
    if (Array.isArray($files)) {
        ServicesFolderFiles = $files;
    }
    else {
        ServicesFolderFiles = $.file.readDirectory($folder);
    }
    for (const ServiceFile of ServicesFolderFiles) {
        const fullPath = `${$folder}/${ServiceFile}`;
        if ($.file.isDirectory(fullPath)) {
            LoadServicesInDirectory(fullPath);
        }
        else {
            try {
                const services = require(fullPath);
                if (typeof services !== "object" || (typeof services === "object" && !Object.keys(services).length)) {
                    return $.logErrorAndExit(`No ControllerService object found in {${ServiceFile}}`);
                }
                const namespace = services.namespace || "";
                if (services.hasOwnProperty("namespace")) {
                    if (ServicesInMemory.has(namespace)) {
                        return $.logErrorAndExit(`Namespace: {${namespace}} cannot be used because it conflicts with a registered ControllerService name.`);
                    }
                    delete services.namespace;
                }
                if (namespace.length) {
                    for (const service of Object.keys(services)) {
                        const key = `${namespace}.${service}`;
                        ServicesInMemory.set(key, services[service]);
                    }
                }
                else {
                    ServicesInMemory.merge(services);
                }
            }
            catch (e) {
                $.logErrorAndExit(e);
            }
        }
    }
};
if (AutoLoadPaths && $.file.isDirectory(ServicesFolder)) {
    if (AutoLoadPaths === "all") {
        LoadServicesInDirectory(ServicesFolder);
    }
    else if (Array.isArray(AutoLoadPaths)) {
        LoadServicesInDirectory(ServicesFolder, AutoLoadPaths);
    }
}
class ControllerEngine {
    static getMiddlewares($middleware, method, route, processOnly = false) {
        const middlewareKeys = Object.keys($middleware);
        const middlewares = [];
        for (let i = 0; i < middlewareKeys.length; i++) {
            let middleware = middlewareKeys[i];
            let middlewareFile = [];
            let middlewareMethod = $middleware[middleware];
            if (middleware.substring(0, 1) === "@") {
                const oldMiddlewareMethod = middlewareMethod;
                middlewareMethod = middleware.substring(1);
                middleware = oldMiddlewareMethod;
            }
            if (typeof middleware === "string") {
                if (middleware.indexOf(".") > 0) {
                    const m = middleware.split(".");
                    m[0] = lodash_1.default.upperFirst(m[0]) + "Middleware";
                    middlewareFile = m;
                }
                else {
                    middleware = lodash_1.default.upperFirst(middleware) + "Middleware";
                    middlewareFile = [middleware, "allow"];
                }
            }
            if (middlewareMethod === "*"
                || middlewareMethod === method
                || (Array.isArray(middlewareMethod) && middlewareMethod.includes(method))) {
                let path = route.path;
                if (typeof path === "string") {
                    if (path.trim() === "/") {
                        path = new RegExp("^\/$");
                    }
                }
                if (Array.isArray(middleware)) {
                    for (const item of middleware) {
                        if (typeof item === "function") {
                            middlewares.push(item);
                        }
                    }
                }
                else if (typeof middleware === "function") {
                    middlewares.push(middleware);
                }
                else {
                    middlewares.push((0, MiddlewareEngine_1.default)(middlewareFile[0], middlewareFile[1], route, processOnly));
                }
            }
        }
        return middlewares;
    }
    controller() {
    }
    ;
    constructor(route, controller, method, isPath) {
        return this.processController(route, controller, method, isPath);
    }
    processController(route, controller, method, isPath) {
        let controllerName = (typeof controller.name === "string") ? controller.name : "__UNNAMED_CONTROLLER__";
        const controllerIsObject = typeof controller === "object";
        const controllerIsService = controller instanceof ControllerService_1.default;
        const $handlerArguments = [];
        let errorHandler = null;
        const handlerArguments = () => lodash_1.default.clone($handlerArguments);
        if (controllerIsObject && typeof method === "string") {
            if (controllerIsService) {
                controller = controller.getClone();
            }
            controllerName = controller.name || controllerName;
            errorHandler = controller.e || null;
            if (controller.hasOwnProperty(method) && typeof controller[method] === "object") {
                const actions = controller[method];
                const config = controller.__extend__ || { services: {} };
                if (actions.hasOwnProperty("e")) {
                    errorHandler = actions.e;
                    delete actions.e;
                }
                const DefinedServices = config.services || {};
                const serviceKeys = Object.keys(actions);
                const services = {};
                for (const service of serviceKeys) {
                    const serviceIsDefined = DefinedServices.hasOwnProperty(service);
                    const serviceIsInjected = typeof actions[service] === "function";
                    if (!serviceIsDefined && !serviceIsInjected) {
                        if (ServicesInMemory.has(service)) {
                            DefinedServices[service] = ServicesInMemory.get(service);
                        }
                        else {
                            $.logErrorAndExit(`Service {${service}} does not exists in {${controllerName}}`);
                        }
                    }
                    if (serviceIsInjected) {
                        services[service] = [actions[service]];
                    }
                    else {
                        services[service] = DefinedServices[service];
                    }
                }
                $handlerArguments.push(actions);
                $handlerArguments.push({
                    services,
                });
                if (errorHandler) {
                    $handlerArguments.push(errorHandler);
                }
            }
        }
        if (typeof controller === "function") {
            if (isPath) {
                return controller(express_1.default.Router({ mergeParams: true }));
            }
            else if (typeof controller.extendsMainController !== "boolean") {
                method = controller;
            }
        }
        let m;
        if (typeof method === "string") {
            m = method;
        }
        else {
            m = "";
        }
        let useController = controller;
        if (typeof method !== "function") {
            if (!controllerIsObject && typeof controller[method] !== "function") {
                useController = new controller();
            }
            if (typeof useController[method] !== "function" && typeof useController[method] !== "object") {
                return $.logErrorAndExit(`Method: {${method}} does not exist in controller: {${controllerName}}`);
            }
        }
        if (typeof errorHandler !== "function") {
            errorHandler = () => false;
        }
        return async (req, res, next) => {
            const http = new ExtendedRequestEngine_1.default(req, res, next, route);
            const mockErrorHandler = (...args) => {
                return errorHandler(http, ...args);
            };
            try {
                let boot = http.state.get("boot", {});
                if (typeof controller.boot === "function") {
                    try {
                        boot = await controller.boot(http, mockErrorHandler);
                    }
                    catch (e) {
                        return http.newError(e).view({
                            error: {
                                in: `Controller Boot Method: [${controllerName}]`,
                                html: `Error in Controller Boot Method:  <code>${controllerName}</code>`,
                                log: e.stack,
                            },
                        });
                    }
                }
                if (!(boot instanceof http_1.ServerResponse)) {
                    try {
                        let $return;
                        const typeOfControllerMethod = typeof useController[method];
                        if (typeof method === "function") {
                            $return = await method(http, mockErrorHandler, boot);
                        }
                        else if (typeof method === "string") {
                            if (typeOfControllerMethod === "function") {
                                $return = await useController[method](http, boot, mockErrorHandler);
                            }
                            else if (typeOfControllerMethod === "object") {
                                const processArgs = handlerArguments();
                                processArgs.unshift(boot);
                                processArgs.unshift(http);
                                $return = await (0, ProcessServices_1.default)(...processArgs);
                            }
                        }
                        if ($return && !($return instanceof http_1.ServerResponse) && (typeof $return === "string" || typeof $return === "object")) {
                            if (typeof $return === "string") {
                                return res.send($return);
                            }
                            else {
                                return res.json($return);
                            }
                        }
                    }
                    catch (e) {
                        return http.newError(e).view({
                            error: {
                                in: `Controller: [${controllerName}], Method: [${m}]`,
                                html: `Error in Controller:  <code>${controllerName}</code>, Method: <code>${m}</code>`,
                                log: e.stack,
                            },
                        });
                    }
                }
            }
            catch (e) {
                $.logError(e);
            }
        };
    }
}
const Controller = (route, method = null, processOnly = false) => {
    let $controller = undefined;
    let controllerPath = null;
    let isPath = false;
    let isObjectController = false;
    if (typeof route === "object") {
        if (route.hasOwnProperty("controller")) {
            $controller = route.controller;
        }
        if (route.hasOwnProperty("children")) {
            isPath = true;
        }
    }
    if (typeof $controller === "string" && $controller.includes("@")) {
        const controllerAndMethod = (0, internals_fn_1.parseControllerString)($controller);
        $controller = controllerAndMethod.controller;
        method = controllerAndMethod.method;
        controllerPath = $.use.controller(Path_1.default.addProjectFileExtension($controller));
        $controller = require(controllerPath);
    }
    if (typeof $controller === "object") {
        isObjectController = true;
    }
    if ($controller && !isPath && typeof $controller !== "function" && !isObjectController) {
        if (typeof $controller === "string") {
            return $.logErrorAndExit("Controller: {" + $controller + "} not found!");
        }
        return $.logErrorAndExit("Controller not found!");
    }
    const use = (arg) => ExtendedRequestEngine_1.default.expressify(arg);
    let controllerMiddleware;
    if ($controller && route !== undefined) {
        const isControllerService = $controller instanceof ControllerService_1.default;
        if (isControllerService || isObjectController) {
            const ctrl = isControllerService ? $controller.controller : $controller;
            if (typeof ctrl.middleware === "function") {
                controllerMiddleware = ctrl.middleware({ use });
            }
            else if (typeof ctrl.middlewares === "object") {
                controllerMiddleware = ctrl.middlewares;
            }
        }
        else if (typeof $controller.middleware === "function") {
            controllerMiddleware = $controller.middleware({ use });
        }
    }
    let middlewares = [];
    if (controllerMiddleware) {
        middlewares = ControllerEngine.getMiddlewares(controllerMiddleware, method, route, processOnly);
    }
    if (processOnly) {
        return { middlewares, $controller, method };
    }
    const $method = new ControllerEngine(route, $controller, method, isPath);
    return {
        middlewares,
        method: $method,
    };
};
module.exports = Controller;
