"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const http_1 = require("http");
const ControllerServiceError_1 = __importDefault(require("./ControllerServiceError"));
module.exports = async (http, boot, requestServices, config, error) => {
    const DefinedServices = config.services || {};
    const completedServices = {};
    const serviceKeys = Object.keys(requestServices);
    const serviceErrorHandler = (...args) => {
        return new ControllerServiceError_1.default(args);
    };
    for (const serviceKey of serviceKeys) {
        const options = {
            boot,
            http,
            services: completedServices,
            error: serviceErrorHandler
        };
        const action = DefinedServices[serviceKey];
        let serviceResult;
        if (Array.isArray(action)) {
            serviceResult = await action[0](options);
        }
        else {
            serviceResult = await DefinedServices[serviceKey](requestServices[serviceKey], options);
        }
        if (serviceResult instanceof ControllerServiceError_1.default && error) {
            serviceResult = error(http, ...serviceResult.args);
        }
        if (serviceResult instanceof http_1.ServerResponse) {
            return serviceResult;
        }
        completedServices[serviceKey] = serviceResult;
    }
    const completedServiceKeys = Object.keys(completedServices);
    const lastCompletedService = completedServiceKeys[completedServiceKeys.length - 1];
    if (lastCompletedService) {
        return completedServices[lastCompletedService];
    }
};
