"use strict";
const statusCodes = {
    400: "Bad Request",
    401: "Unauthorized",
    404: "Not Found",
    500: "Server Error",
};
class ErrorEngine {
    constructor(http, e) {
        this.http = http;
        this.error = e;
    }
    hasCustomErrorHandler() {
        return typeof this.http["onError"] === "function";
    }
    ;
    view(data, status = 500) {
        if (this.hasCustomErrorHandler()) {
            return this.http["onError"](this.error, data.error);
        }
        return this.http.status(status).renderViewFromEngine("__errors/index", {
            ...data,
            statusCode: status,
            statusCodes
        });
    }
    pageNotFound() {
        const req = this.http.req;
        const error = {
            title: `404 Error!`,
            message: `<code>${req.method}: ${req.url}</code> <br><br> Route not found!`,
        };
        return this.view({ error }, 404);
    }
}
module.exports = ErrorEngine;
