"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const os_1 = __importDefault(require("os"));
class InXpresserError extends Error {
    constructor(message) {
        super(message);
        this.date = new Date();
        this.dateString = new Date().toLocaleDateString('en-US', {
            day: 'numeric',
            weekday: 'short',
            year: 'numeric',
            month: 'short',
            hour: 'numeric',
            minute: 'numeric',
            second: 'numeric',
        });
        this.name = 'Error';
        Error.captureStackTrace(this, this.constructor);
    }
    static try(fn) {
        return this.tryOrCatch(fn, (e) => {
            console.log(e);
        });
    }
    static tryOrCatch(fn, handleError) {
        try {
            return fn();
        }
        catch (e) {
            if (handleError)
                return handleError(this.use(e));
            throw this.use(e);
        }
    }
    static use(e) {
        const error = new this(e.message);
        const stack = e.stack.split(os_1.default.EOL);
        stack.splice(0, 1);
        stack.unshift(error.stack.split(os_1.default.EOL)[0]);
        error.stack = stack.join(os_1.default.EOL);
        return error;
    }
}
module.exports = InXpresserError;
