"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const index_1 = require("../../index");
const Path_1 = __importDefault(require("../Helpers/Path"));
const Emitter_1 = __importDefault(require("./Emitter"));
const $ = (0, index_1.getInstance)();
const DefinedEvents = {};
const EventsPath = $.path.events();
const $useDotJson = $.engineData.get("UseDotJson");
const excludedFiles = $useDotJson.get("exclude.events.files", []);
const excludedFolders = $useDotJson.get("exclude.events.folders", []);
Path_1.default.addProjectFileExtension(excludedFiles);
function readEventsDirectory(parent) {
    const Events = fs_1.default.readdirSync(parent);
    for (let i = 0; i < Events.length; i++) {
        const event = Events[i];
        const fullPath = parent + "/" + event;
        const shortPath = fullPath.replace(EventsPath + "/", "");
        if (fs_1.default.lstatSync(fullPath).isDirectory()) {
            if (!excludedFolders.includes(event)) {
                readEventsDirectory(fullPath);
            }
        }
        else {
            if (!excludedFiles.includes(event)) {
                let event;
                try {
                    event = require(fullPath);
                }
                catch (e) {
                    $.logPerLine([
                        { error: `Error in ${shortPath}` },
                        { error: e },
                    ]);
                }
                if (typeof event === "object") {
                    const eventKeys = Object.keys(event);
                    const namespace = event.namespace || Path_1.default.removeProjectFileExtension(shortPath);
                    for (let j = 0; j < eventKeys.length; j++) {
                        const eventKey = eventKeys[j];
                        if (typeof event[eventKey] === "function") {
                            let name = namespace;
                            if (eventKey !== "index") {
                                name = `${namespace}.${eventKey}`;
                            }
                            DefinedEvents[name] = event[eventKey];
                        }
                    }
                }
            }
        }
    }
}
if (fs_1.default.existsSync(EventsPath)) {
    $.ifConsole(() => {
        if ($.config.get('artisan.loadEvents')) {
            readEventsDirectory(EventsPath);
        }
    }, () => {
        readEventsDirectory(EventsPath);
    });
}
$.events = (0, Emitter_1.default)(DefinedEvents);
