"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const build_url_1 = __importDefault(require("build-url"));
const index_1 = require("../../index");
const moment_1 = __importDefault(require("moment"));
const $ = (0, index_1.getInstance)();
const helpers = {
    urlBuilder: build_url_1.default,
    url($path = "", $query = {}) {
        let url = "";
        const server = $.config.get('server');
        if ($path.substring(0, 1) === "/")
            $path = $path.substring(1);
        if (server.baseUrl.length) {
            url = server.baseUrl + $path;
        }
        else {
            let d = server.domain;
            let p = server.protocol;
            if (server.includePortInUrl && (server.port !== 80 && server.port !== 443)) {
                d = d + ":" + server.port;
            }
            if ($.config.get("server.ssl.enabled", false)) {
                p = "https";
            }
            url = p + "://" + d + server.root + $path;
        }
        if (Object.keys($query).length) {
            url = (0, build_url_1.default)(url, {
                queryParams: $query,
            });
        }
        return url;
    },
    route($route, $keys = [], $query = {}, $includeUrl = true) {
        if (typeof $query === "boolean") {
            $includeUrl = $query;
            $query = {};
        }
        if (!Array.isArray($keys)) {
            $keys = [$keys];
        }
        const routes = $.routerEngine.nameToPath();
        if (typeof routes[$route] !== "undefined") {
            let path = routes[$route];
            if (path.slice(-1) === "*" && !$keys.length) {
                return path.slice(0, path.length - 1);
            }
            const hasRegex = path.match(new RegExp('[|&;$%@"<>()+:,*]'));
            if (Array.isArray(hasRegex) && hasRegex.length) {
                const findKeys = new RegExp("[*]|(:[a-z_A-Z]+)", "g");
                const HasKeys = path.match(findKeys);
                if (Array.isArray(HasKeys) && HasKeys.length) {
                    let counter = 0;
                    const replacer = (...args) => {
                        if (args[0] === "*" && !$keys.length) {
                            counter++;
                            return "*";
                        }
                        const key = $keys[counter] || (args[0] === "*" ? "*" : "_??_");
                        counter++;
                        return key;
                    };
                    path = path.replace(findKeys, replacer);
                }
            }
            return $includeUrl ? helpers.url(path, $query) : path;
        }
        return $includeUrl ? helpers.url($route, $query) : $route;
    },
    config($config, $default) {
        return $.config.get($config, $default);
    },
    mix(file) {
        let mix;
        const localVariableName = "laravelMixManifest";
        if (file.substring(0, 1) !== "/") {
            file = "/" + file;
        }
        if ($.engineData.has(localVariableName)) {
            mix = $.engineData.get(localVariableName);
        }
        else {
            const mixFile = $.path.base("public/mix-manifest.json");
            if ($.file.exists(mixFile)) {
                mix = require(mixFile);
                $.engineData.set(localVariableName, mix);
            }
        }
        if (typeof mix[file] !== "undefined") {
            file = mix[file];
        }
        return helpers.url(file);
    },
    env(key, $default) {
        return $.env(key, $default);
    },
    randomStr(length = 10) {
        let i;
        let possible;
        let text;
        text = "";
        possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
        i = 0;
        while (i < length) {
            text += possible.charAt(Math.floor(Math.random() * possible.length));
            i++;
        }
        return text;
    },
    randomArray(length = 5) {
        let i = 0;
        const newArray = [];
        while (i < length) {
            newArray.push(i);
            i++;
        }
        return newArray;
    },
    randomInteger(min, max) {
        return Math.floor(Math.random() * (max - min + 1)) + min;
    },
    now() {
        return (0, moment_1.default)().format($.config.get('date.format'));
    },
    today() {
        return (0, moment_1.default)().format($.config.get('date.format'));
    },
    toDate(date, format) {
        if (!format) {
            format = $.config.get('date.format');
        }
        if (!date) {
            date = helpers.now();
        }
        return (0, moment_1.default)(date, format);
    },
    timeAgo(date, format) {
        return helpers.toDate(date, format).fromNow();
    },
};
module.exports = helpers;
