"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const lodash_1 = __importDefault(require("lodash"));
const index_1 = require("../../index");
const inbuilt_fn_1 = require("../Functions/inbuilt.fn");
const os_1 = __importDefault(require("os"));
const $ = (0, index_1.getInstance)();
const isDebugEnabled = $.config.sync('debug.enabled', true);
const depWarnings = $.config.sync('debug.deprecationWarnings');
$.log = (...args) => {
    if (!args.length) {
        return console.log("");
    }
    args.unshift(chalk_1.default.white("=>"));
    if (args.length === 2 && typeof args[1] === "string") {
        return console.log(chalk_1.default.cyanBright(...args));
    }
    return console.log(...args);
};
$.logCalmly = (...args) => {
    if (!args.length) {
        return console.log("");
    }
    args.unshift(chalk_1.default.white("=>"));
    if (args.length === 2 && typeof args[1] === "string") {
        return console.log(chalk_1.default.white(...args));
    }
    return console.log(...args);
};
$.logDeprecated = (since, removedAt, message, hasStack = true) => {
    if (Array.isArray(message)) {
        const m = message;
        for (const i in m) {
            if (m[i] === null) {
                m[i] = os_1.default.EOL;
            }
            else {
                m[i] += ' ';
            }
        }
        message = message.join('').trim();
    }
    const mustaches = (0, inbuilt_fn_1.touchMyMustache)(message);
    if (mustaches.length) {
        mustaches.forEach(m => {
            const withoutMustache = m.replace('{{', '').replace('}}', '');
            message = message.replace(m, chalk_1.default.cyan(withoutMustache));
        });
    }
    const config = depWarnings.sync;
    if (isDebugEnabled.sync && config.enabled) {
        console.log(chalk_1.default.gray('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>'));
        console.log(chalk_1.default.whiteBright(`!! DEPRECATED ALERT !!`));
        if (hasStack && config.showStack) {
            console.log(chalk_1.default.white(os_1.default.EOL + message));
            console.trace();
            console.log();
        }
        else {
            console.log(chalk_1.default.white(os_1.default.EOL + message + os_1.default.EOL));
        }
        console.log(chalk_1.default.whiteBright(`Since: `) + chalk_1.default.white(since) + `, ` +
            chalk_1.default.whiteBright(`To be removed: `) + chalk_1.default.white(removedAt));
        console.log(chalk_1.default.gray('<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<'));
    }
};
$.logInfo = (...args) => {
    if (!args.length) {
        return console.log("");
    }
    args.unshift("=>");
    if (args.length === 2 && typeof args[1] === "string") {
        return console.log(chalk_1.default.magentaBright(...args));
    }
    return console.log(...args);
};
$.logSuccess = (...args) => {
    if (!args.length) {
        return console.log("");
    }
    args.unshift("✔✔");
    if (args.length === 2 && typeof args[1] === "string") {
        return console.log(chalk_1.default.greenBright(...args));
    }
    return console.log(...args);
};
$.logWarning = (...args) => {
    if (!args.length) {
        return console.log("");
    }
    args.unshift("!!");
    if (args.length === 2 && typeof args[1] === "string") {
        return console.log(chalk_1.default.yellow(...args));
    }
    return console.log(...args);
};
$.logIfNotConsole = (...args) => {
    if (!$.options.isConsole) {
        $.log(...args);
    }
};
$.logAndExit = (...args) => {
    if (args.length) {
        $.log(...args);
    }
    return $.exit();
};
$.logError = (error, exit = false) => {
    if (error instanceof index_1.InXpresserError) {
        console.log(chalk_1.default.redBright(error.stack));
        if (error.dateString) {
            $.logWarning('Occurred: ' + error.dateString);
        }
    }
    else if (error instanceof Error) {
        console.log(chalk_1.default.redBright(error.stack ? error.stack : error));
    }
    else if (typeof error === "string") {
        console.log(chalk_1.default.redBright(error));
    }
    else {
        console.error(error);
    }
    if (exit) {
        return $.exit();
    }
};
$.logErrorAndExit = (error) => {
    return $.logError(error, true);
};
$.logPerLine = ($logs = [], $spacePerLine = false) => {
    console.log();
    for (let i = 0; i < $logs.length; i++) {
        const $log = $logs[i];
        if (typeof $log === "function") {
            $log();
        }
        else if (typeof $log === "object") {
            const key = Object.keys($log)[0];
            $["log" + lodash_1.default.upperFirst(key)]($log[key]);
        }
        else {
            if (typeof $log === "string" && !$log.length) {
                $.log();
            }
            else {
                $.log($log);
            }
        }
        if ($spacePerLine) {
            $.log();
        }
    }
    console.log();
};
$.env = (key, $default) => {
    if (typeof process.env[key] === "undefined") {
        return $default;
    }
    return process.env[key];
};
