"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Path_1 = __importDefault(require("../Helpers/Path"));
const index_1 = require("../../index");
const path_1 = __importDefault(require("path"));
const packageName = "xpresser";
const $ = (0, index_1.getInstance)();
const paths = $.config.get('paths');
const baseFiles = paths.base + "/";
const backendFiles = Path_1.default.resolve(paths.backend);
$.path = {
    resolve: (path, resolve = true) => {
        return Path_1.default.resolve(path, resolve);
    },
    base: (path = "", returnRequire = false) => {
        if (path[0] === "/") {
            path = path.substring(1);
        }
        const base = path_1.default.resolve(baseFiles + path);
        return returnRequire ? require(base) : base;
    },
    backend: (path = "", returnRequire = false) => {
        if (path[0] === "/") {
            path = path.substring(1);
        }
        const backend = path_1.default.resolve(backendFiles + "/" + path);
        return returnRequire ? require(backend) : backend;
    },
    storage: (path = "") => {
        if (path[0] === "/") {
            path = path.substring(1);
        }
        return path_1.default.resolve(paths.storage + "/" + path);
    },
    frameworkStorageFolder: (path) => {
        if (path === undefined) {
            path = "";
        }
        if (path[0] === "/") {
            path = path.substring(1);
        }
        return $.path.storage("framework/" + path);
    },
    engine: (path = "", returnRequire = false, refresh = false) => {
        const dataKey = "XpresserPath";
        let EnginePath;
        if (!refresh && $.engineData.has(dataKey)) {
            EnginePath = $.engineData.get(dataKey);
        }
        else {
            if (typeof paths.engine === "string") {
                EnginePath = Path_1.default.resolve(paths.engine) + "/";
            }
            else {
                EnginePath = Path_1.default.resolve([paths.npm, packageName, "src"]) + "/";
            }
            $.engineData.set(dataKey, EnginePath);
        }
        if (path[0] === "/") {
            path = path.substring(1);
        }
        const engine = EnginePath + path;
        return returnRequire ? require(engine) : engine;
    },
    events: (path = "", returnRequire = false) => {
        if (path[0] === "/") {
            path = path.substring(1);
        }
        const event = Path_1.default.resolve([paths.events, path]);
        return returnRequire ? require(event) : event;
    },
    controllers: (path = "", returnRequire = false) => {
        if (path[0] === "/") {
            path = path.substring(1);
        }
        const controller = Path_1.default.resolve([paths.controllers, path]);
        return returnRequire ? require(controller) : controller;
    },
    middlewares: (path = "", returnRequire = false) => {
        if (path[0] === "/") {
            path = path.substring(1);
        }
        const middleware = Path_1.default.resolve([paths.middlewares, path]);
        return returnRequire ? require(middleware) : middleware;
    },
    models: (path = "", returnRequire = false) => {
        if (path[0] === "/") {
            path = path.substring(1);
        }
        const model = Path_1.default.resolve([paths.models, path]);
        return returnRequire ? require(model) : model;
    },
    views: (path = "") => {
        if (path[0] === "/") {
            path = path.substring(1);
        }
        return Path_1.default.resolve([paths.views, path]);
    },
    jsonConfigs: (path = "") => {
        if (path[0] === "/") {
            path = path.substring(1);
        }
        return Path_1.default.resolve([paths.jsonConfigs, path]);
    },
    configs: (path = "", returnRequire = false) => {
        if (path[0] === "/") {
            path = path.substring(1);
        }
        const config = Path_1.default.resolve([paths.configs, path]);
        return returnRequire ? require(config) : config;
    },
    node_modules(path = "") {
        let currentNodeModules;
        try {
            currentNodeModules = require.resolve('xpresser')
                .replace('xpresser\\dist\\index.js', '')
                .replace('xpresser/dist/index.js', '');
        }
        catch (e) {
            currentNodeModules = Path_1.default.resolve([paths.npm, path]);
        }
        return currentNodeModules + path;
    }
};
