"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const index_1 = require("../index");
const fs_extra_1 = __importDefault(require("fs-extra"));
const $ = (0, index_1.getInstance)();
$.file = {
    fs() {
        return fs_1.default;
    },
    fsExtra() {
        return fs_extra_1.default;
    },
    size($path) {
        try {
            return fs_1.default.statSync($path).size;
        }
        catch (e) {
            return 0;
        }
    },
    isFile($path) {
        try {
            return fs_1.default.statSync($path).isFile();
        }
        catch (e) {
            return false;
        }
    },
    isSymbolicLink($path) {
        try {
            return fs_1.default.statSync($path).isSymbolicLink();
        }
        catch (e) {
            return false;
        }
    },
    isDirectory($path) {
        try {
            return fs_1.default.statSync($path).isDirectory();
        }
        catch (e) {
            return false;
        }
    },
    get($path, $options) {
        const fileExists = $.file.exists($path);
        if (!fileExists) {
            return false;
        }
        return fs_1.default.readFileSync($path, $options);
    },
    read($path, $options) {
        return $.file.get($path, $options);
    },
    readDirectory($path, $options) {
        return this.getDirectory($path, $options);
    },
    getDirectory($path, $options) {
        const fileExists = $.file.exists($path);
        if (!fileExists) {
            return false;
        }
        return fs_1.default.readdirSync($path, $options);
    },
    exists($path, $returnList = false) {
        if (Array.isArray($path)) {
            const files = $path;
            const filesFound = [];
            for (const file of files) {
                const fileExists = $.file.exists(file);
                if (!$returnList && !fileExists) {
                    return false;
                }
                if (fileExists) {
                    filesFound.push(file);
                }
            }
            return $returnList ? filesFound : true;
        }
        else {
            try {
                return fs_1.default.existsSync($path);
            }
            catch (e) {
                return false;
            }
        }
    },
    delete($path, $returnList = false) {
        if (Array.isArray($path)) {
            const paths = $path;
            const pathsDeleted = [];
            for (const path of paths) {
                const pathExists = $.file.delete(path);
                if (!$returnList && !pathExists) {
                    return false;
                }
                if (pathExists) {
                    pathsDeleted.push(path);
                }
            }
            return $returnList ? pathsDeleted : true;
        }
        else {
            try {
                fs_1.default.unlinkSync($path);
                return true;
            }
            catch (e) {
                return false;
            }
        }
    },
    deleteDirectory($path, options) {
        if (Array.isArray($path)) {
            const paths = $path;
            const returnList = options && options.returnList;
            const pathsDeleted = [];
            for (const path of paths) {
                const deleted = $.file.deleteDirectory(path, options);
                if (!(returnList) && !deleted) {
                    return false;
                }
                if (deleted)
                    pathsDeleted.push(path);
            }
            return returnList ? pathsDeleted : true;
        }
        else {
            try {
                let rmDirOptions = undefined;
                if (options) {
                    const { returnList, ...others } = options;
                    rmDirOptions = others;
                }
                fs_1.default.rmSync($path, rmDirOptions);
                return true;
            }
            catch (e) {
                return false;
            }
        }
    },
    readJson($path, fileExists = false) {
        if (!fileExists && !fs_1.default.existsSync($path)) {
            throw Error(`$.file.readJson: Path (${$path}) does not exists.`);
        }
        try {
            const file = fs_1.default.readFileSync($path).toString();
            return JSON.parse(file);
        }
        catch (e) {
            throw Error(`$.file.readJson: Error parsing json file (${$path})`);
        }
    },
    saveToJson($path, $content, $options = {}) {
        $options = Object.assign({
            checkIfFileExists: true,
            replacer: null,
            space: 2
        }, $options);
        if ($options.checkIfFileExists && !fs_1.default.existsSync($path)) {
            throw Error(`$.file.saveToJson: Path (${$path}) does not exists.`);
        }
        try {
            fs_1.default.writeFileSync($path, JSON.stringify($content, $options.replacer, $options.space));
            return true;
        }
        catch (e) {
            console.log(e);
            throw Error(`$.file.saveToJson: Error saving data to json file (${$path})`);
        }
    },
    makeDirIfNotExist($path, $isFile = false) {
        if ($isFile) {
            $path = path_1.default.dirname($path);
        }
        if (!fs_1.default.existsSync($path)) {
            try {
                fs_1.default.mkdirSync($path, { recursive: true });
            }
            catch (e) {
                $.logErrorAndExit(e);
            }
        }
        return $path;
    }
};
