"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const fs_1 = __importDefault(require("fs"));
const lodash_1 = __importDefault(require("lodash"));
const index_1 = require("../../index");
const handlebars_1 = __importDefault(require("handlebars"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const pluralize_1 = __importDefault(require("pluralize"));
const consoleColors_obj_1 = __importDefault(require("../Objects/consoleColors.obj"));
const Path_1 = __importDefault(require("../Helpers/Path"));
const $ = (0, index_1.getInstance)();
const isTinker = typeof $.options.isTinker === "boolean" && $.options.isTinker;
const FILE_EXTENSION = $.config.get("project.fileExtension", ".js");
const afterLastSlash = (str) => {
    if (str.includes("/")) {
        const parts = str.split("/");
        return lodash_1.default.upperFirst(parts[parts.length - 1]);
    }
    return lodash_1.default.upperFirst(str);
};
module.exports = {
    logThis(...args) {
        if (isTinker) {
            args.unshift(consoleColors_obj_1.default.fgMagenta);
        }
        else {
            args.unshift(consoleColors_obj_1.default.fgCyan);
        }
        args.push(consoleColors_obj_1.default.reset);
        return $.log(...args);
    },
    logThisAndExit(...args) {
        if (isTinker) {
            return this.logThis(...args);
        }
        else {
            this.logThis(...args);
            $.logAndExit();
        }
    },
    factory(file, $data = {}) {
        if ($.isTypescript()) {
            const tsFile = file.replace('.hbs', '.ts.hbs');
            if (fs_1.default.existsSync(tsFile)) {
                file = tsFile;
            }
        }
        const source = fs_1.default.readFileSync(file).toString();
        const template = handlebars_1.default.compile(source);
        return template($data);
    },
    copyFromFactoryToApp($for, $name, $to, $data = {}, addPrefix = true) {
        let $factory;
        if (Array.isArray($for)) {
            $factory = $for[1];
            $for = $for[0];
        }
        if (!fs_1.default.existsSync($to)) {
            Path_1.default.makeDirIfNotExist($to);
        }
        if (!$name.toLowerCase().includes($for)) {
            if (addPrefix && $for !== "model") {
                $name = $name + lodash_1.default.upperFirst($for);
            }
        }
        if ($name.includes('/')) {
            const names = $name.split('/');
            const lastPath = lodash_1.default.upperFirst(names.pop());
            names.push(lastPath);
            $name = names.join('/');
        }
        else {
            $name = lodash_1.default.upperFirst($name);
        }
        $to = $to + "/" + $name + FILE_EXTENSION;
        if (fs_1.default.existsSync($to)) {
            return this.logThisAndExit($name + " already exists!");
        }
        Path_1.default.makeDirIfNotExist($to, true);
        const factoryName = $factory ? $factory : $for;
        let $from = $.path.engine("Factory/" + factoryName + ".hbs");
        let customFactoryFile = $.config.get(`artisan.factory.${factoryName}`);
        if (customFactoryFile) {
            if (!customFactoryFile.includes('.hbs')) {
                return this.logThisAndExit(`Custom factory file defined for ${factoryName} is not a (.hbs) file`);
            }
            customFactoryFile = Path_1.default.resolve(customFactoryFile);
            if (!$.file.exists(customFactoryFile)) {
                return this.logThisAndExit(`Custom factory file defined for ${factoryName} does not exist.`);
            }
            $from = customFactoryFile;
        }
        $data['name'] = afterLastSlash($name);
        fs_1.default.writeFileSync($to, this.factory($from, $data));
        this.logThis($name + " created successfully.");
        this.logThis("located @ " + $to.replace($.path.base(), ""));
    },
    copyFolder($from, $to) {
        fs_extra_1.default.copySync($from, $to);
        $.logAndExit("Views folder published successful.");
    },
    pluralize(str = "") {
        if (!str.trim().length) {
            return str;
        }
        return (0, pluralize_1.default)(str);
    },
    singular(str) {
        return pluralize_1.default.singular(str);
    },
};
