"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initializeTypescriptFn = exports.parseControllerString = void 0;
const index_1 = require("../../index");
const Path_1 = __importDefault(require("../Helpers/Path"));
const path_1 = __importDefault(require("path"));
const $ = (0, index_1.getInstance)();
function parseControllerString(controllerString) {
    const split = controllerString.split("@");
    let startIndex = 0;
    if (split.length > 2)
        startIndex = split.length - 2;
    const method = split[startIndex + 1];
    return {
        controller: controllerString.replace(`@${method}`, ''),
        method
    };
}
exports.parseControllerString = parseControllerString;
function initializeTypescriptFn(filename, run) {
    if (!filename)
        throw Error(`isTypescript: requires __filename as argument.`);
    const isTypeScriptFile = filename.slice(-3) === '.ts';
    const tsBaseFolder = isTypeScriptFile ? $.path.base() : path_1.default.resolve($.path.resolve(`base://../`));
    $.engineData.set('tsBaseFolder', tsBaseFolder);
    if (isTypeScriptFile) {
        $.config.set('project.fileExtension', '.ts');
        if ($.config.get("paths.routesFile") === "backend://routes.js") {
            $.config.set('paths.routesFile', 'backend://routes.ts');
        }
    }
    if (!isTypeScriptFile && !$.file.exists(Path_1.default.resolve($.config.get('paths.npm')))) {
        try {
            $.config.set('paths.npm', $.path.node_modules());
            $.path.engine('', false, true);
        }
        catch (e) {
            $.logError('Path to xpresser engine files maybe missing, point {config.paths.npm} to your node_modules folder.');
        }
    }
    if (!isTypeScriptFile) {
        const routesFile = $.config.get('paths.routesFile');
        if (routesFile.includes('.ts')) {
            $.config.set('paths.routesFile', routesFile.replace('.ts', '.js'));
        }
        let jsonConfigs = $.config.get('paths.jsonConfigs');
        jsonConfigs = $.path.resolve(jsonConfigs);
        jsonConfigs = jsonConfigs.replace($.path.base(), tsBaseFolder + '/');
        $.config.set('paths.jsonConfigs', jsonConfigs);
        let viewsPath = $.config.get('paths.views');
        viewsPath = $.path.resolve(viewsPath);
        const tsViewsPath = viewsPath.replace($.path.base(), tsBaseFolder + '/');
        if ($.file.exists(tsViewsPath) && !$.file.exists(viewsPath)) {
            $.config.set('paths.views', tsViewsPath);
        }
    }
    if (typeof run === "function") {
        run(isTypeScriptFile ? false : {
            ts: {
                baseFolder: tsBaseFolder
            }
        });
    }
    return $;
}
exports.initializeTypescriptFn = initializeTypescriptFn;
