"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const path_1 = __importDefault(require("path"));
const index_1 = require("../../index");
const fs_1 = __importDefault(require("fs"));
const $ = (0, index_1.getInstance)();
const pathHelpers = {
    base: "base://",
    backend: "backend://",
    frontend: "frontend://",
    npm: "npm://",
    public: "public://",
};
class PathHelper {
    static path() {
        return path_1.default;
    }
    static resolve($path, $resolve = true) {
        if (Array.isArray($path)) {
            for (let i = 0; i < $path.length; i++) {
                const $pathElement = $path[i];
                if ($pathElement.slice(-1) === "/") {
                    $path[i] = $pathElement.substring(0, $pathElement.length - 1);
                }
            }
            $path = $path.join("/");
        }
        if ($path.includes('.js.js')) {
            $path = $path.replace('.js.js', '.js');
        }
        if ($.isTypescript()) {
            if ($path.includes('.js.ts')) {
                $path = $path.replace('.js.ts', '.js');
            }
            if ($path.includes('.ts.js')) {
                $path = $path.replace('.js.js', '.ts');
            }
        }
        if ($path.indexOf("://") >= 0) {
            const $splitPath = $path.split("://");
            if (pathHelpers.hasOwnProperty($splitPath[0])) {
                if (!$splitPath[1]) {
                    $splitPath[1] = "";
                }
                return PathHelper.helperToPath($splitPath);
            }
        }
        if ($path.slice(-1) === "/") {
            $path = $path.slice(0, -1);
        }
        return $resolve ? (p => path_1.default.resolve(p))($path) : $path;
    }
    static helperToPath([$helper, $path]) {
        const config = $.config.get('paths');
        if ($helper === "base") {
            return config.base + "/" + $path;
        }
        else if ($helper === "npm") {
            return PathHelper.resolve([config.npm, $path]);
        }
        else {
            if (config[$helper]) {
                return PathHelper.resolve([config[$helper], $path]);
            }
            return $.path.base(`${$helper}/${$path}`);
        }
    }
    frameworkStorage(path) {
        if (path === undefined) {
            path = "";
        }
        if (path[0] === "/") {
            path = path.substring(1);
        }
        return $.path.storage("framework/" + path);
    }
    static makeDirIfNotExist($path, $isFile = false) {
        if ($isFile === true) {
            $path = path_1.default.dirname($path);
        }
        if (!fs_1.default.existsSync($path)) {
            try {
                fs_1.default.mkdirSync($path, { recursive: true });
            }
            catch (e) {
                $.logErrorAndExit(e);
            }
        }
        return $path;
    }
    static addProjectFileExtension(files, useExtension, clone = false) {
        if (Array.isArray(files)) {
            let array;
            if (clone) {
                array = [...files];
            }
            else {
                array = files;
            }
            for (let i = 0; i < array.length; i++) {
                const file = array[i];
                array[i] = PathHelper.addProjectFileExtension(file);
            }
            return array;
        }
        else {
            const jsExt = ".js";
            useExtension = (useExtension && useExtension.length) ? useExtension : $.config.get('project.fileExtension');
            const ext = useExtension ? useExtension : jsExt;
            const hasExtInName = files.slice(-ext.length) === ext;
            if (ext !== jsExt &&
                !hasExtInName &&
                files.slice(-jsExt.length) === jsExt) {
                return files;
            }
            return hasExtInName ? files : files + ext;
        }
    }
    static removeProjectFileExtension(files, clone = false) {
        if (Array.isArray(files)) {
            let array;
            if (clone) {
                array = [...files];
            }
            else {
                array = files;
            }
            for (let i = 0; i < array.length; i++) {
                const file = array[i];
                array[i] = this.removeProjectFileExtension(file);
            }
            return array;
        }
        else {
            return files.substring(0, files.length - $.config.get('project.fileExtension').length);
        }
    }
    static getExtension(path, withDot = true) {
        const dots = path.split('.');
        return dots.length > 1 ? ((withDot ? '.' : '') + dots.pop()) : undefined;
    }
}
module.exports = PathHelper;
