"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const lodash_1 = __importDefault(require("lodash"));
const ExtendedRequestEngine_1 = __importDefault(require("./Plugins/ExtendedRequestEngine"));
const index_1 = require("../index");
const $ = (0, index_1.getInstance)();
const projectFileExtension = $.config.get('project.fileExtension', '').substring(1);
const MiddlewareEngine = (middlewarePath, action, route, processOnly = false) => {
    if (middlewarePath.indexOf(".") > 0) {
        const m = middlewarePath.split(".");
        if (m[1] !== projectFileExtension) {
            middlewarePath = m[0];
            action = m[1];
        }
    }
    middlewarePath = lodash_1.default.upperFirst(middlewarePath);
    const middleware = $.use.middleware(middlewarePath, false);
    if (middleware === false) {
        return $.logErrorAndExit("Middleware: " + middlewarePath + " not found!");
    }
    if (typeof middleware !== "object" && typeof middleware !== "function") {
        return $.logErrorAndExit("No Middleware found in: " + middlewarePath);
    }
    if (typeof middleware === "object") {
        if (!action) {
            action = "allow";
        }
        if (typeof middleware[action] === "undefined") {
            return $.logErrorAndExit("MethodWithHttp {" + action + "} not found in middleware: " + middlewarePath);
        }
    }
    if (processOnly) {
        return middleware;
    }
    return ExtendedRequestEngine_1.default.expressify(http => {
        try {
            if (typeof middleware === "function") {
                return middleware(http);
            }
            else {
                return middleware[action](http);
            }
        }
        catch (e) {
            return http.newError(e).view({
                error: {
                    in: `Middleware: [${middlewarePath}]`,
                    html: `Error in Middleware <code>${middlewarePath}</code>`,
                    log: e.stack,
                },
            });
        }
    });
};
module.exports = MiddlewareEngine;
