"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const fs_1 = __importDefault(require("fs"));
const lodash_1 = __importDefault(require("lodash"));
const index_1 = require("../../index");
const moment_1 = __importDefault(require("moment"));
const Path_1 = __importDefault(require("../Helpers/Path"));
const $ = (0, index_1.getInstance)();
const PluginLockDataPath = $.path.jsonConfigs("plugins-lock.json");
let PluginLockData = $.objectCollection();
let UpdatePluginLockData = false;
Path_1.default.makeDirIfNotExist(PluginLockDataPath, true);
if (fs_1.default.existsSync(PluginLockDataPath)) {
    const lockData = require(PluginLockDataPath);
    PluginLockData = $.objectCollection(lockData);
}
module.exports = ($plugin) => {
    const $pluginNamespaces = $.engineData.get("PluginEngine:namespaces", {});
    const $pluginNamespaceKeys = Object.keys($pluginNamespaces);
    let $pluginData = null;
    for (let i = 0; i < $pluginNamespaceKeys.length; i++) {
        const $pluginNamespace = $pluginNamespaces[$pluginNamespaceKeys[i]];
        if ($pluginNamespace.plugin === $plugin) {
            $pluginData = $pluginNamespace;
            break;
        }
    }
    if ($pluginData === null) {
        return $.logErrorAndExit(`Plugin: ${$plugin} not found.`);
    }
    if (!PluginLockData.has($plugin)) {
        PluginLockData.set($plugin, {});
    }
    const $thisPluginLockData = PluginLockData.newInstanceFrom($plugin);
    if ($thisPluginLockData.get("installed", false)) {
        return $.logPerLine([
            { info: `Plugin: ${$plugin} is already installed!` },
            { errorAndExit: "" },
        ]);
    }
    $.logInfo(`Installing ${$plugin}...`);
    if ($pluginData.hasOwnProperty("migrations")) {
        const $migrationLockData = $thisPluginLockData.newInstanceFrom("migrations");
        const $migrationsFolder = $pluginData.migrations;
        const $migrationFiles = fs_1.default.readdirSync($migrationsFolder);
        if ($migrationFiles.length) {
            Path_1.default.makeDirIfNotExist($.path.base("migrations"));
        }
        for (let i = 0; i < $migrationFiles.length; i++) {
            const $migrationFile = $migrationFiles[i];
            const $splitMigrationFileName = $migrationFile.split("_");
            let $newMigrationFilePath = $migrationFile;
            if (!isNaN(Number($splitMigrationFileName[0]))) {
                $splitMigrationFileName[0] = (0, moment_1.default)(new Date())
                    .format("YMMDHmmss")
                    .toString() + "_(" + $pluginData.namespace.toLowerCase() + ")";
            }
            const $newMigrationFile = $splitMigrationFileName.join("_");
            $newMigrationFilePath = $.path.base("migrations/" + $newMigrationFile);
            const $migrationFileFullPath = $migrationsFolder + "/" + $migrationFile;
            if (!$migrationLockData.has($newMigrationFile)) {
                fs_1.default.copyFileSync($migrationFileFullPath, $newMigrationFilePath);
                $.logInfo(`Moved migration: ${$newMigrationFile}`);
                const $data = {
                    migrations: {
                        [$migrationFile]: $newMigrationFile,
                    },
                };
                $thisPluginLockData.merge($data);
                UpdatePluginLockData = true;
            }
        }
    }
    if ($pluginData.hasOwnProperty("models")) {
        const $models = fs_1.default.readdirSync($pluginData.models);
        for (let i = 0; i < $models.length; i++) {
            const $modelFile = $models[i];
            let $model = $models[i];
            const $newModel = lodash_1.default.upperFirst($pluginData.namespace) + "/" + $model;
            $model = $pluginData.models + "/" + $model;
            const $newModelFullPath = $.path.models($newModel);
            Path_1.default.makeDirIfNotExist($newModelFullPath, true);
            if (!fs_1.default.existsSync($newModelFullPath)) {
                fs_1.default.copyFileSync($model, $newModelFullPath);
            }
            $.logInfo(`Moved model: ${$newModel}`);
            const $data = {
                models: {
                    [$modelFile]: $newModel,
                },
            };
            $thisPluginLockData.merge($data);
            UpdatePluginLockData = true;
        }
    }
    const $pluginIsInstalled = $thisPluginLockData.get("installed", false);
    if (!$pluginIsInstalled) {
        try {
            const pluginInit = require($pluginData.path);
            if (typeof pluginInit.install === "function") {
                pluginInit.install();
            }
            $thisPluginLockData.set("installed", true);
            UpdatePluginLockData = true;
        }
        catch (e) {
            $.logError($plugin);
            $.logErrorAndExit(e);
        }
    }
    if (UpdatePluginLockData) {
        fs_1.default.writeFileSync(PluginLockDataPath, JSON.stringify(PluginLockData.all(), null, 2));
        $.logInfo("Updated plugins-lock.json");
    }
    $.logInfo("Installation Complete!");
};
