"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const lodash_1 = __importDefault(require("lodash"));
const index_1 = require("../index");
const internals_fn_1 = require("./Functions/internals.fn");
const router_fn_1 = require("./Functions/router.fn");
const Path_1 = __importDefault(require("./Helpers/Path"));
const MiddlewareEngine_1 = __importDefault(require("./MiddlewareEngine"));
const $ = (0, index_1.getInstance)();
const AllRoutesKey = "RouterEngine:allRoutes";
const NameToRoute = {};
const ProcessedRoutes = [];
const ControllerStringCache = {};
class RouterEngine {
    static allRoutes() {
        return $.engineData.get(AllRoutesKey);
    }
    static addToRoutes(route) {
        if (typeof route.routes !== "undefined" && Array.isArray(route.routes)) {
            const allRoutes = $.router.routes;
            $.router.routes = lodash_1.default.concat(allRoutes, route.routes);
            $.engineData.set(AllRoutesKey, $.router.routes);
        }
    }
    static allProcessedRoutes($format, $key = "path") {
        if ($format === "array") {
            const routesArray = [];
            for (let i = 0; i < ProcessedRoutes.length; i++) {
                const processedRoute = ProcessedRoutes[i];
                const routeArray = [
                    processedRoute.method.toUpperCase(),
                    processedRoute.path,
                    processedRoute.name || null,
                ];
                routesArray.push(routeArray);
            }
            return routesArray;
        }
        else if ($format === "key") {
            const routesArray = [];
            for (let i = 0; i < ProcessedRoutes.length; i++) {
                const processedRoute = ProcessedRoutes[i];
                routesArray.push(processedRoute[$key]);
            }
            return routesArray;
        }
        return ProcessedRoutes;
    }
    static namedRoutes(format = false) {
        if (format !== false) {
            const names = Object.keys(NameToRoute);
            const newFormat = {};
            for (let i = 0; i < names.length; i++) {
                const name = names[i];
                const route = NameToRoute[name];
                newFormat[route.method + " " + route.path] = "{" + route.name + "} ===> " + route.controller;
            }
            if (typeof format === "string" && format === "json") {
                return JSON.stringify(newFormat, null, 2);
            }
            return newFormat;
        }
        return NameToRoute;
    }
    static nameToPath(returnKey = "path") {
        const localVariableName = "RouterEngine:nameToPath";
        if ($.engineData.has(localVariableName)) {
            return $.engineData.get(localVariableName);
        }
        const names = Object.keys(NameToRoute);
        const newRoutes = {};
        for (let i = 0; i < names.length; i++) {
            const name = names[i];
            newRoutes[name] = NameToRoute[name][returnKey];
        }
        if (returnKey !== "path") {
            return newRoutes;
        }
        $.engineData.set(localVariableName, newRoutes);
        return newRoutes;
    }
    static nameToUrl() {
        const localVariableName = "RouterEngine:nameToUrl";
        if ($.engineData.has(localVariableName)) {
            return $.engineData.get(localVariableName);
        }
        const routes = RouterEngine.nameToPath();
        const names = Object.keys(routes);
        const newRoutes = {};
        for (let i = 0; i < names.length; i++) {
            const name = names[i];
            newRoutes[name] = $.helpers.route(name, [], false);
        }
        $.engineData.set(localVariableName, newRoutes);
        return newRoutes;
    }
    static processRoutes(routes = null, parent) {
        const Controller = require("./ControllerEngine");
        if (!Array.isArray(routes)) {
            routes = RouterEngine.allRoutes();
        }
        for (let i = 0; i < routes.length; i++) {
            let route = routes[i].data;
            let routeAsPath = Array.isArray(route.children) ? route : false;
            let nameWasGenerated = false;
            if (routeAsPath && parent) {
                if (parent.children) {
                    if (typeof routeAsPath.as === "string" && typeof parent.as === "string" && routeAsPath.as.substring(0, 1) === ".") {
                        routeAsPath.as = parent.as + routeAsPath.as;
                    }
                    lodash_1.default.defaults(routeAsPath, {
                        as: parent.as,
                        controller: parent.controller,
                        useActionsAsName: parent.useActionsAsName,
                    });
                }
            }
            if (typeof route.controller === "string") {
                if (!routeAsPath && (parent === null || parent === void 0 ? void 0 : parent.useActionsAsName) && !route.name) {
                    let nameFromController = route.controller;
                    if (nameFromController.includes("@")) {
                        let splitName = nameFromController.split("@");
                        nameFromController = splitName[splitName.length - 1];
                    }
                    route.name = lodash_1.default.snakeCase(nameFromController);
                    nameWasGenerated = true;
                }
            }
            if ((parent === null || parent === void 0 ? void 0 : parent.as) && typeof route.name === "string" && route.name.substring(0, 1) !== "/") {
                if (route.path === "" && nameWasGenerated) {
                    route.name = parent.as;
                }
                else {
                    route.name = parent.as + "." + route.name;
                }
            }
            if (route.name) {
                if (route.name.substring(0, 1) === "/") {
                    route.name = route.name.substring(1);
                }
                route.name = route.name.toLowerCase();
            }
            if (!routeAsPath && (parent === null || parent === void 0 ? void 0 : parent.controller) && typeof route.controller === "string" && !route.controller.includes("@")) {
                route.controller = parent.controller + "@" + route.controller;
            }
            if (parent === null || parent === void 0 ? void 0 : parent.path) {
                const routePath = $.utils.regExpSourceOrString(route.path);
                const parentPath = $.utils.regExpSourceOrString(parent.path);
                if (route.path instanceof RegExp || parent.path instanceof RegExp) {
                    route.path = new RegExp(`${parentPath}${parentPath !== "/" ? "/" : ""}${routePath}`);
                }
                else {
                    if (routePath.length && parentPath.slice(-1) !== "/" && routePath.substring(0, 1) !== "/") {
                        route.path = "/" + routePath;
                    }
                    route.path = parent.path + route.path;
                }
            }
            if (typeof route.path === "string" && route.path.substring(0, 2) === "//") {
                route.path = route.path.substring(1);
            }
            if (typeof route.name !== "undefined") {
                NameToRoute[route.name] = route;
            }
            if (!routeAsPath && typeof route.controller === "string" && route.controller.includes("@")) {
                let { method, controller } = (0, internals_fn_1.parseControllerString)(route.controller);
                const controllerCacheKey = controller;
                if (ControllerStringCache.hasOwnProperty(controllerCacheKey)) {
                    route.controller = ControllerStringCache[controllerCacheKey] + "@" + method;
                }
                else {
                    let isTypescriptButUsingJsExtension = false;
                    let controllerPath = $.use.controller(Path_1.default.addProjectFileExtension(controller));
                    if (!$.file.exists(controllerPath)) {
                        if (!controller.includes("Controller")) {
                            controllerPath = $.use.controller(Path_1.default.addProjectFileExtension(controller + "Controller"));
                            if (!$.file.exists(controllerPath)) {
                                controllerPath = $.use.controller(Path_1.default.addProjectFileExtension(controller + "Controller", '.js'));
                                if ($.isTypescript() && $.file.exists(controllerPath)) {
                                    isTypescriptButUsingJsExtension = true;
                                }
                                if (!isTypescriptButUsingJsExtension) {
                                    $.logError("Controller: " + [controller, method].join("@") + " not found");
                                    $.logErrorAndExit('Path:', controllerPath);
                                }
                            }
                            controller = controller + "Controller";
                        }
                    }
                    if (isTypescriptButUsingJsExtension) {
                        controller = controller + '.js';
                    }
                    ControllerStringCache[controllerCacheKey] = controller;
                    route.controller = controller + "@" + method;
                }
            }
            const canRegisterRoutes = ($.app && (!$.options.isTinker && !$.options.isConsole));
            if (routeAsPath) {
                if (canRegisterRoutes) {
                    const RegisterMiddleware = (middleware) => {
                        const PathMiddleware = (0, MiddlewareEngine_1.default)(middleware);
                        if (PathMiddleware) {
                            $.app.use(routeAsPath.path, PathMiddleware);
                        }
                    };
                    if (Array.isArray(routeAsPath.middleware)) {
                        routeAsPath.middleware.forEach((element) => {
                            RegisterMiddleware(element);
                        });
                    }
                    else if (typeof routeAsPath.middleware === "string") {
                        RegisterMiddleware(routeAsPath.middleware);
                    }
                }
                if (routeAsPath.children.length) {
                    RouterEngine.processRoutes(routeAsPath.children, routeAsPath);
                }
            }
            else {
                ProcessedRoutes.push({
                    url: (0, router_fn_1.pathToUrl)(route.path),
                    ...route,
                });
                if (canRegisterRoutes) {
                    const controller = Controller(route);
                    $.app[route.method](route.path, controller.middlewares, controller.method);
                }
            }
        }
    }
}
module.exports = RouterEngine;
