"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const index_1 = require("../index");
const Path_1 = __importDefault(require("./Helpers/Path"));
const JobHelper_1 = __importDefault(require("./Console/JobHelper"));
const Commands_1 = __importDefault(require("./Console/Commands"));
const { Commands, Artisan } = Commands_1.default;
const DefinedCommands = {};
const $ = (0, index_1.getInstance)();
const cloneArgs = [...process.argv];
const args = cloneArgs.splice(3);
if (args[args.length - 1] === "--from-xjs-cli") {
    $.options.isFromXjsCli = true;
    args.length = args.length - 1;
    process.argv.length = process.argv.length - 1;
}
let argCommand = args[0];
if (typeof argCommand === "undefined") {
    $.logErrorAndExit("No command provided!");
}
argCommand = argCommand.trim();
let isJobCommand = argCommand.substring(0, 1) === "@";
if (isJobCommand || !Commands.hasOwnProperty(argCommand)) {
    const PluginData = $.engineData.get("PluginEngine:namespaces", {});
    const plugins = Object.keys(PluginData);
    for (const plugin of plugins) {
        const $plugin = PluginData[plugin];
        if ($plugin.hasOwnProperty("commands")) {
            const commands = $plugin["commands"];
            const commandKeys = Object.keys(commands);
            for (const command of commandKeys) {
                Commands[command] = commands[command];
            }
        }
    }
    if (isJobCommand) {
        argCommand = argCommand.substring(1);
        let jobPath = $.path.backend(`jobs/${argCommand}`);
        jobPath = Path_1.default.addProjectFileExtension(jobPath);
        if (!fs_1.default.existsSync(jobPath)) {
            $.logErrorAndExit(`Job: (${argCommand}) does  not exist.`);
        }
        const job = require(jobPath);
        if (typeof job !== "object") {
            $.logErrorAndExit("Job: {" + argCommand + "} did not return object!");
            if (!job.hasOwnProperty("handler")) {
                $.logErrorAndExit("Job: {" + argCommand + "} is not structured properly!");
            }
        }
        DefinedCommands[argCommand] = job;
    }
}
if (typeof Commands[argCommand] === "undefined" && typeof DefinedCommands[argCommand] === "undefined") {
    if ($.options.isTinker) {
        $.log(`Console Command not found: {${argCommand}}`);
    }
    else {
        $.logAndExit(`Command not found: {${argCommand}}`);
    }
}
else {
    const jobHelper = new JobHelper_1.default(argCommand);
    args.splice(0, 1);
    if (typeof Commands[argCommand] === "function") {
        if (argCommand === 'routes') {
            require("./Routes/Loader");
            $.routerEngine.processRoutes($.router.routes);
        }
        Commands[argCommand](args, jobHelper);
    }
    else if (typeof Commands[argCommand] === "string") {
        const command = require(Commands[argCommand]);
        command(args, { artisan: Artisan, helper: jobHelper });
    }
    else if (typeof DefinedCommands[argCommand] === "object") {
        const command = DefinedCommands[argCommand];
        if (typeof command.handler !== "function") {
            $.logAndExit(`Command: {${argCommand}} has no handler method`);
        }
        if (command.use && typeof command.use === "object") {
            const { use } = command;
            if (use.events) {
                require("./Events/Loader");
            }
            if (use.routes) {
                require("./Routes/Loader");
                $.routerEngine.processRoutes($.router.routes);
            }
        }
        try {
            command.handler(args, jobHelper);
        }
        catch (e) {
            $.logPerLine([
                { error: `Error in command: {${argCommand}}` },
                { errorAndExit: e },
            ]);
        }
    }
}
