"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const Path_1 = __importDefault(require("./Helpers/Path"));
const index_1 = require("../index");
const InXpresserError_1 = __importDefault(require("./Errors/InXpresserError"));
const String_1 = __importDefault(require("./Helpers/String"));
const $ = (0, index_1.getInstance)();
let Use;
const PluginNamespaces = $.engineData.get("PluginEngine:namespaces", {});
const projectFileExtension = $.config.get('project.fileExtension');
const UsePath = "UseDotJson";
Use = $.engineData.get(UsePath, $.objectCollection()).all();
if (typeof Use.middlewares === "object") {
    const MiddlewareSuffix = "Middleware";
    const useMiddlewares = Use.middlewares;
    const middlewareKeys = Object.keys(useMiddlewares);
    for (let i = 0; i < middlewareKeys.length; i++) {
        const middlewareKey = middlewareKeys[i];
        let middleware = useMiddlewares[middlewareKey];
        if (middleware.slice(-3) === projectFileExtension) {
            middleware = middleware.slice(0, -3);
        }
        let middlewareRealPath = Path_1.default.resolve(middleware);
        let hasMiddleware = false;
        if ($.file.exists(middlewareRealPath)) {
            hasMiddleware = true;
        }
        else if ($.file.exists(middlewareRealPath + projectFileExtension)) {
            hasMiddleware = true;
        }
        else {
            if ($.file.exists(middlewareRealPath + MiddlewareSuffix + projectFileExtension)) {
                middlewareRealPath = middlewareRealPath + MiddlewareSuffix;
                hasMiddleware = true;
            }
        }
        if (hasMiddleware) {
            const hasSuffix = String_1.default.hasSuffix(middlewareKey, MiddlewareSuffix);
            if (hasSuffix) {
                Use.middlewares[String_1.default.withoutSuffix(middlewareKey, MiddlewareSuffix)] = middlewareRealPath;
                delete Use.middlewares[middlewareKey];
            }
            else {
                Use.middlewares[middlewareKey] = middlewareRealPath;
            }
        }
        else {
            $.logError(`Middleware not found:`);
            $.logErrorAndExit(middleware);
            delete Use.middlewares[middlewareKey];
        }
    }
}
$.engineData.set(UsePath, $.objectCollection(Use));
function parsePath(path, data = {}) {
    const dataKeys = Object.keys(data);
    if (dataKeys.length) {
        for (let i = 0; i < dataKeys.length; i++) {
            const dataKey = dataKeys[i];
            path = path.replace(`{${dataKey}}`, data[dataKey]);
        }
    }
    return path;
}
function fileExistsInPath(file, path, suffix = "") {
    if (suffix.length) {
        const hasSuffix = file.slice(-suffix.length) === suffix;
        if (!hasSuffix) {
            file += suffix;
        }
    }
    let fullPath = parsePath(path, { file });
    fullPath = Path_1.default.resolve(fullPath);
    if (!$.file.exists(fullPath)) {
        file = String_1.default.upperFirst(file);
        if (!$.file.exists(parsePath(path, { file }))) {
            return [false, fullPath];
        }
    }
    return [true, fullPath];
}
class UseEngine {
    static package($package, handleError = true) {
        try {
            return require($package);
        }
        catch (e) {
            return !handleError ? false : $.logErrorAndExit(`Package {${[$package]}} not found in node_modules.`);
        }
    }
    static file(path) {
        const fullPath = $.path.backend("{file}" + projectFileExtension);
        const [hasPath, realPath] = fileExistsInPath(path, fullPath);
        if (!hasPath) {
            throw Error(`File ${realPath} does not exist!`);
        }
        return require(realPath);
    }
    static model(model, handleError = true) {
        let fullPath = Path_1.default.resolve($.config.get('paths.models')) + "/{file}";
        fullPath = Path_1.default.addProjectFileExtension(fullPath);
        const [hasPath, realPath] = fileExistsInPath(model, fullPath);
        if (!hasPath) {
            if (!handleError) {
                return false;
            }
            else {
                throw new InXpresserError_1.default(`Model ${realPath} does not exists`);
            }
        }
        return require(realPath);
    }
    static middleware(middleware, handleError = true, suffix = true) {
        if (typeof Use.middlewares === "object") {
            const useMiddlewares = Use.middlewares;
            const middleWithoutSuffix = String_1.default.withoutSuffix(middleware, "Middleware");
            if (useMiddlewares.hasOwnProperty(middleWithoutSuffix)) {
                return require(useMiddlewares[middleWithoutSuffix]);
            }
        }
        const fullPath = $.path.backend("middlewares/{file}" + projectFileExtension);
        const [hasPath, realPath] = fileExistsInPath(middleware, fullPath, suffix ? "Middleware" : "");
        if (!hasPath) {
            return !handleError ? false : $.logErrorAndExit(new Error(`Middleware ${realPath} does not exits`));
        }
        return require(realPath);
    }
    static controller(controller) {
        if (controller.indexOf("::") > 0) {
            const $splitController = controller.split("::");
            const $pluginNamespace = $splitController[0];
            if (PluginNamespaces.hasOwnProperty($pluginNamespace)) {
                const plugin = PluginNamespaces[$pluginNamespace];
                if (plugin.paths && plugin.paths.hasOwnProperty("controllers")) {
                    return plugin.paths.controllers + "/" + $splitController[1];
                }
            }
        }
        return $.path.controllers(controller);
    }
    static router() {
        return $.router;
    }
}
module.exports = UseEngine;
